Token

Enum Token 

pub enum Token<'a> {
Show 397 variants Error, Always, And, Assign, Begin, Buf, Bufif0, Bufif1, Case, Casex, Casez, Cmos, Deassign, Default, Defparam, Disable, Edge, Else, End, Endcase, Endfunction, Endmodule, Endprimitive, Endspecify, Endtable, Endtask, Event, For, Force, Forever, Fork, Function, Highz0, Highz1, If, Ifnone, Initial, Inout, Input, Integer, Join, Large, Macromodule, Medium, Module, Nand, Negedge, Nmos, Nor, Not, Notif0, Notif1, Or, Output, Parameter, Pmos, Posedge, Primitive, Pull0, Pull1, Pulldown, Pullup, Rcmos, Real, Realtime, Reg, Release, Repeat, Rnmos, Rpmos, Rtran, Rtranif0, Rtranif1, Scalared, Small, Specify, Specparam, Strong0, Strong1, Supply0, Supply1, Table, Task, Time, Tran, Tranif0, Tranif1, Tri, Tri0, Tri1, Triand, Trior, Trireg, Vectored, Wait, Wand, Weak0, Weak1, While, Wire, Wor, Xnor, Xor, Automatic, Cell, Config, Design, Endconfig, Endgenerate, Generate, Genvar, Incdir, Include, Instance, Liblist, Library, Localparam, Noshowcancelled, PulsestyleOndetect, PulsestyleOnevent, Showcancelled, Signed, Unsigned, Use, Uwire, Alias, AlwaysComb, AlwaysFf, AlwaysLatch, Assert, Assume, Before, Bind, Bins, Binsof, Bit, Break, Byte, Chandle, Class, Clocking, Const, Constraint, Context, Continue, Cover, Covergroup, Coverpoint, Cross, Dist, Do, Endclass, Endclocking, Endgroup, Endinterface, Endpackage, Endprogram, Endproperty, Endsequence, Enum, Expect, Export, Extends, Extern, Final, FirstMatch, Foreach, Forkjoin, Iff, IgnoreBins, IllegalBins, Import, Inside, Int, Interface, Intersect, JoinAny, JoinNone, Local, Logic, Longint, Matches, Modport, New, Null, Package, Packed, Priority, Program, Property, Protected, Pure, Rand, Randc, Randcase, Randsequence, Ref, Return, Sequence, Shortint, Shortreal, Solve, Static, String, Struct, Super, Tagged, This, Throughout, Timeprecision, Timeunit, Type, Typedef, Union, Unique, Var, Virtual, Void, WaitOrder, Wildcard, With, Within, AcceptOn, Checker, Endchecker, Eventually, Global, Implies, Let, Nexttime, RejectOn, Restrict, SAlways, SEventually, SNexttime, SUntil, SUntilWith, Strong, SyncAcceptOn, SyncRejectOn, Unique0, Until, UntilWith, Untyped, Weak, Implements, Interconnect, Nettype, Soft, DirUnderscoreFile, DirUnderscoreLine, DirBeginKeywords, DirCelldefine, DirDefaultNettype, DirDefine, DirElse, DirElsif, DirEndKeywords, DirEndcelldefine, DirEndif, DirIfdef, DirIfndef, DirInclude, DirLine, DirNounconnectedDrive, DirPragma, DirResetall, DirTimescale, DirUnconnectedDrive, DirUndef, DirUndefineall, Plus, Minus, Exclamation, Quest, Tilde, Amp, TildeAmp, Pipe, TildePipe, Caret, TildeCaret, CaretTilde, Star, Slash, Percent, EqEq, ExclEq, PlusEq, MinusEq, StarEq, SlashEq, PercentEq, AmpEq, PipeEq, CaretEq, EqEqEq, ExclEqEq, EqEqQuest, ExclEqQuest, AmpAmp, AmpAmpAmp, PipePipe, StarStar, Lt, LtEq, Gt, GtEq, GtGt, LtLt, GtGtEq, LtLtEq, GtGtGt, LtLtLt, GtGtGtEq, LtLtLtEq, MinusGt, MinusGtGt, LtMinusGt, PlusPlus, MinusMinus, PlusColon, MinusColon, PlusSlashMinus, PlusPercentMinus, Paren, EParen, Bracket, EBracket, Brace, EBrace, Colon, SColon, Apost, Comma, Period, Pound, Dollar, At, AtAt, Eq, ColonColon, ColonEq, ColonSlash, PoundPound, PoundMinusPound, PoundEqPound, EqGt, StarGt, PipeMinusGt, PipeEqGt, Bslash, Std, PathpulseDollar, Option, TypeOption, Randomize, Sample, OneStep, DollarSetup, DollarHold, DollarSetuphold, DollarRecovery, DollarRemoval, DollarRecrem, DollarSkew, DollarTimeskew, DollarFullskew, DollarPeriod, DollarWidth, DollarNochange, DollarRoot, DollarUnit, DollarFatal, DollarError, DollarWarning, DollarInfo, OnelineComment(&'a str), BlockComment(&'a str), UnsignedNumber(&'a str), FixedPointNumber(&'a str), BinaryNumber(&'a str), OctalNumber(&'a str), DecimalNumber(&'a str), HexNumber(&'a str), ScientificNumber(&'a str), UnbasedUnsizedLiteral(&'a str), SystemTfIdentifier(&'a str), SimpleIdentifier(&'a str), EscapedIdentifier(&'a str), PreprocessorIdentifier(&'a str), TextMacro(&'a str), StringLiteral(&'a str), PreprocessorStringLiteral(&'a str), TripleQuoteStringLiteral(&'a str), PreprocessorTripleQuoteStringLiteral(&'a str), Newline,
}
A single syntactic token for a SystemVerilog source file

Variants



Error

A lexer error



Always



And



Assign



Begin



Buf



Bufif0



Bufif1



Case



Casex



Casez



Cmos



Deassign



Default



Defparam



Disable



Edge



Else



End



Endcase



Endfunction



Endmodule



Endprimitive



Endspecify



Endtable



Endtask



Event



For



Force



Forever



Fork



Function



Highz0



Highz1



If



Ifnone



Initial



Inout



Input



Integer



Join



Large



Macromodule



Medium



Module



Nand



Negedge



Nmos



Nor



Not



Notif0



Notif1



Or



Output



Parameter



Pmos



Posedge



Primitive



Pull0



Pull1



Pulldown



Pullup



Rcmos



Real



Realtime



Reg



Release



Repeat



Rnmos



Rpmos



Rtran



Rtranif0



Rtranif1



Scalared



Small



Specify



Specparam



Strong0



Strong1



Supply0



Supply1



Table



Task



Time



Tran



Tranif0



Tranif1



Tri



Tri0



Tri1



Triand



Trior



Trireg



Vectored



Wait



Wand



Weak0



Weak1



While



Wire



Wor



Xnor



Xor



Automatic



Cell



Config



Design



Endconfig



Endgenerate



Generate



Genvar



Incdir



Include



Instance



Liblist



Library



Localparam



Noshowcancelled



PulsestyleOndetect



PulsestyleOnevent



Showcancelled



Signed



Unsigned



Use



Uwire



Alias



AlwaysComb



AlwaysFf



AlwaysLatch



Assert



Assume



Before



Bind



Bins



Binsof



Bit



Break



Byte



Chandle



Class



Clocking



Const



Constraint



Context



Continue



Cover



Covergroup



Coverpoint



Cross



Dist



Do



Endclass



Endclocking



Endgroup



Endinterface



Endpackage



Endprogram



Endproperty



Endsequence



Enum



Expect



Export



Extends



Extern



Final



FirstMatch



Foreach



Forkjoin



Iff



IgnoreBins



IllegalBins



Import



Inside



Int



Interface



Intersect



JoinAny



JoinNone



Local



Logic



Longint



Matches



Modport



New



Null



Package



Packed



Priority



Program



Property



Protected



Pure



Rand



Randc



Randcase



Randsequence



Ref



Return



Sequence



Shortint



Shortreal



Solve



Static



String



Struct



Super



Tagged



This



Throughout



Timeprecision



Timeunit



Type



Typedef



Union



Unique



Var



Virtual



Void



WaitOrder



Wildcard



With



Within



AcceptOn



Checker



Endchecker



Eventually



Global



Implies



Let



Nexttime



RejectOn



Restrict



SAlways



SEventually



SNexttime



SUntil



SUntilWith



Strong



SyncAcceptOn



SyncRejectOn



Unique0



Until



UntilWith



Untyped



Weak



Implements



Interconnect



Nettype



Soft



DirUnderscoreFile



DirUnderscoreLine



DirBeginKeywords



DirCelldefine



DirDefaultNettype



DirDefine



DirElse



DirElsif



DirEndKeywords



DirEndcelldefine



DirEndif



DirIfdef



DirIfndef



DirInclude



DirLine



DirNounconnectedDrive



DirPragma



DirResetall



DirTimescale



DirUnconnectedDrive



DirUndef



DirUndefineall



Plus



Minus



Exclamation



Quest



Tilde



Amp



TildeAmp



Pipe



TildePipe



Caret



TildeCaret



CaretTilde



Star



Slash



Percent



EqEq



ExclEq



PlusEq



MinusEq



StarEq



SlashEq



PercentEq



AmpEq



PipeEq



CaretEq



EqEqEq



ExclEqEq



EqEqQuest



ExclEqQuest



AmpAmp



AmpAmpAmp



PipePipe



StarStar



Lt



LtEq



Gt



GtEq



GtGt



LtLt



GtGtEq



LtLtEq



GtGtGt



LtLtLt



GtGtGtEq



LtLtLtEq



MinusGt



MinusGtGt



LtMinusGt



PlusPlus



MinusMinus



PlusColon



MinusColon



PlusSlashMinus



PlusPercentMinus



Paren



EParen



Bracket



EBracket



Brace



EBrace



Colon



SColon



Apost



Comma



Period



Pound



Dollar



At



AtAt



Eq



ColonColon



ColonEq



ColonSlash



PoundPound



PoundMinusPound



PoundEqPound



EqGt



StarGt



PipeMinusGt



PipeEqGt



Bslash



Std



PathpulseDollar



Option



TypeOption



Randomize



Sample



OneStep



DollarSetup



DollarHold



DollarSetuphold



DollarRecovery



DollarRemoval



DollarRecrem



DollarSkew



DollarTimeskew



DollarFullskew



DollarPeriod



DollarWidth



DollarNochange



DollarRoot



DollarUnit



DollarFatal



DollarError



DollarWarning



DollarInfo



OnelineComment(&'a str)



BlockComment(&'a str)



UnsignedNumber(&'a str)



FixedPointNumber(&'a str)



BinaryNumber(&'a str)



OctalNumber(&'a str)



DecimalNumber(&'a str)



HexNumber(&'a str)



ScientificNumber(&'a str)



UnbasedUnsizedLiteral(&'a str)



SystemTfIdentifier(&'a str)



SimpleIdentifier(&'a str)



EscapedIdentifier(&'a str)



PreprocessorIdentifier(&'a str)



TextMacro(&'a str)



StringLiteral(&'a str)



PreprocessorStringLiteral(&'a str)



TripleQuoteStringLiteral(&'a str)



PreprocessorTripleQuoteStringLiteral(&'a str)



Newline

Implementations

Source

impl<'a> Token<'a>

pub fn is_directive(&self) -> bool

Whether the token represents a compiler directive

pub fn as_str(&self) -> &'static str

A string representation of the token (usually the literal matching text)