pub enum PreprocessorWarning<'a> {
NotPreviouslyDefinedMacro((&'a str, Span<'a>)),
RedefinedMacro((&'a str, Span<'a>, Span<'a>)),
}Expand description
A warning encountered during preprocessing
Warnings reflect an irregularity in the source code, but are still well-defined and allow preprocessing to continue
Variants
NotPreviouslyDefinedMacro((&'a str, Span<'a>))
Attempted to `undef a macro that had no previous definition
let source = "
`undef TEST
";
let input = lex(source, "test.v").tokens();
let preprocess_result = preprocess(
&mut TokenIterator::new(input.into_iter()),
&mut state,
&cache,
);
assert!(matches!(preprocess_result, Ok(_)));
assert!(matches!(state.warnings.first(), Some(PreprocessorWarning::NotPreviouslyDefinedMacro(_))))RedefinedMacro((&'a str, Span<'a>, Span<'a>))
A redefinition of a text macro that was previously defined
let source = "
`define TEST definition_one
`define TEST definition_two
";
let input = lex(source, "test.v").tokens();
let preprocess_result = preprocess(
&mut TokenIterator::new(input.into_iter()),
&mut state,
&cache,
);
assert!(matches!(preprocess_result, Ok(_)));
assert!(matches!(state.warnings.first(), Some(PreprocessorWarning::RedefinedMacro(_))))Trait Implementations
Sourceimpl<'a> Clone for PreprocessorWarning<'a>
impl<'a> Clone for PreprocessorWarning<'a>
Sourcefn clone(&self) -> PreprocessorWarning<'a>
fn clone(&self) -> PreprocessorWarning<'a>
1.0.0 (const: unstable) · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
source. Read moreSourceimpl<'a> Debug for PreprocessorWarning<'a>
impl<'a> Debug for PreprocessorWarning<'a>
Sourceimpl<'s> From<&PreprocessorWarning<'s>> for Report<'s, (String, Range<usize>)>
impl<'s> From<&PreprocessorWarning<'s>> for Report<'s, (String, Range<usize>)>
Sourcefn from(s: &PreprocessorWarning<'s>) -> Self
fn from(s: &PreprocessorWarning<'s>) -> Self
Sourceimpl<'a> PartialEq for PreprocessorWarning<'a>
impl<'a> PartialEq for PreprocessorWarning<'a>
Sourcefn eq(&self, other: &PreprocessorWarning<'a>) -> bool
fn eq(&self, other: &PreprocessorWarning<'a>) -> bool
self and other values to be equal, and is used by ==.impl<'a> StructuralPartialEq for PreprocessorWarning<'a>