Module report

Module report 

Reporting errors from parsing

Structs

Report
A printable error report

Enums

ReportKind
A type that defines the kind of report being produced.

Traits

Cache
A trait implemented by Source caches.
Sources
A cache of sources that can be used for printing reports