Report

Struct Report 

pub struct Report { /* private fields */ }
A printable error report

These include detailed information about the location of the error, and are printed with file snippets to assist the user

Implementations

Source

impl<'a> Report

pub fn new<C, M>( kind: ReportKind<'static>, span: &Span<'a>, code: C, msg: M, ) -> Self
where C: Display, M: ToString,

Create a new Report indicating a particular location with a message

This does not label/print the location; to do so, use Report::with_label

pub fn with_label<M>( self, span: &Span<'a>, kind: ReportKind<'static>, msg: M, ) -> Self
where M: ToString,

Adds a label to the Report

A label includes a Span to highlight, as well as a message to attach at that location

pub fn print<C>(self, source_cache: &mut C) -> Result<()>
where C: Cache<String>,

Print the report to stdout

pub fn eprint<C>(self, source_cache: &mut C) -> Result<()>
where C: Cache<String>,

Print the report to stderr

pub fn write<C, W>(self, source_cache: &mut C, target: W) -> Result<()>
where C: Cache<String>, W: Write,

Writes the report (without color) to a target with the std::io::Write trait