Sources

Trait Sources 

pub trait Sources {
    // Required method
    fn sources(self) -> impl Cache<String>;
}
A cache of sources that can be used for printing reports

Currently, this is implemented for iterators over (file_name, file_content) tuples; this should be the common use case, with others implemented at their own risk

Required Methods

fn sources(self) -> impl Cache<String>

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementors

Source

impl<I, S> Sources for I
where I: IntoIterator<Item = (String, S)>, S: AsRef<str>,