pub type LexerResult<'a> = (Result<Token<'a>, String>, Span<'a>);
A single result from the lexer
This is either a valid token, or a (possible) explanation of what went wrong, if identifiable, along with the associated Span
Span