pub struct Report { /* private fields */ }Expand description
A printable error report
These include detailed information about the location of the error, and are printed with file snippets to assist the user
Implementations
Sourceimpl<'a> Report
impl<'a> Report
Sourcepub fn new<C, M>(
kind: ReportKind<'static>,
span: &Span<'a>,
code: C,
msg: M,
) -> Self
pub fn new<C, M>( kind: ReportKind<'static>, span: &Span<'a>, code: C, msg: M, ) -> Self
Create a new Report indicating a particular location with a message
This does not label/print the location; to do so, use Report::with_label
Sourcepub fn with_label<M>(
self,
span: &Span<'a>,
kind: ReportKind<'static>,
msg: M,
) -> Selfwhere
M: ToString,
pub fn with_label<M>(
self,
span: &Span<'a>,
kind: ReportKind<'static>,
msg: M,
) -> Selfwhere
M: ToString,